@@ -6,32 +6,33 @@
  ============================================================================
  */
 
-#define _XOPEN_SOURCE 500
-#include <stdio.h>
-#include <stdlib.h>
-#include <string.h>
-#include <unistd.h>
-#include <termios.h>
-#include <stdbool.h>
-#include <stdint.h>
-#include <time.h>
-#include <signal.h>
+#define _XOPEN_SOURCE 500 // for: usleep
+#include <stdio.h>	  // defines: printf, puts, getchar
+#include <stdlib.h>	  // defines: EXIT_SUCCESS
+#include <string.h>	  // defines: strcmp
+#include <unistd.h>	  // defines: STDIN_FILENO, usleep
+#include <termios.h>	  // defines: termios, TCSANOW, ICANON, ECHO
+#include <stdbool.h>	  // defines: true, false
+#include <stdint.h>	  // defines: uint8_t, uint32_t
+#include <time.h>	  // defines: time
+#include <signal.h>	  // defines: signal, SIGINT
 
 #define SIZE 4
-uint32_t score = 0;
-uint8_t scheme = 0;
 
-void getColors(uint8_t value, uint8_t *foreground, uint8_t *background)
+// this function receives 2 pointers (indicated by *) so it can set their values
+void getColors(uint8_t value, uint8_t scheme, uint8_t *foreground, uint8_t *background)
 {
 	uint8_t original[] = {8, 255, 1, 255, 2, 255, 3, 255, 4, 255, 5, 255, 6, 255, 7, 255, 9, 0, 10, 0, 11, 0, 12, 0, 13, 0, 14, 0, 255, 0, 255, 0};
 	uint8_t blackwhite[] = {232, 255, 234, 255, 236, 255, 238, 255, 240, 255, 242, 255, 244, 255, 246, 0, 248, 0, 249, 0, 250, 0, 251, 0, 252, 0, 253, 0, 254, 0, 255, 0};
 	uint8_t bluered[] = {235, 255, 63, 255, 57, 255, 93, 255, 129, 255, 165, 255, 201, 255, 200, 255, 199, 255, 198, 255, 197, 255, 196, 255, 196, 255, 196, 255, 196, 255, 196, 255};
 	uint8_t *schemes[] = {original, blackwhite, bluered};
+	// modify the 'pointed to' variables (using a * on the left hand of the assignment)
 	*foreground = *(schemes[scheme] + (1 + value * 2) % sizeof(original));
 	*background = *(schemes[scheme] + (0 + value * 2) % sizeof(original));
+	// alternatively we could have returned a struct with two variables
 }
 
-uint8_t getNumberLength(uint32_t number)
+uint8_t getDigitCount(uint32_t number)
 {
 	uint8_t count = 0;
 	do
@@ -42,44 +43,46 @@ uint8_t getNumberLength(uint32_t number)
 	return count;
 }
 
-void drawBoard(uint8_t board[SIZE][SIZE])
+void drawBoard(uint8_t board[SIZE][SIZE], uint8_t scheme, uint32_t score)
 {
 	uint8_t x, y, fg, bg;
-	printf("\033[H");
+	printf("\033[H"); // move cursor to 0,0
 	printf("2048.c %17d pts\n\n", score);
 	for (y = 0; y < SIZE; y++)
 	{
 		for (x = 0; x < SIZE; x++)
 		{
-			getColors(board[x][y], &fg, &bg);
+			// send the addresses of the foreground and background variables,
+			// so that they can be modified by the getColors function
+			getColors(board[x][y], scheme, &fg, &bg);
 			printf("\033[38;5;%d;48;5;%dm", fg, bg); // set color
 			printf("       ");
-			printf("\033[m"); // reset
+			printf("\033[m"); // reset all modes
 		}
 		printf("\n");
 		for (x = 0; x < SIZE; x++)
 		{
-			getColors(board[x][y], &fg, &bg);
+			getColors(board[x][y], scheme, &fg, &bg);
 			printf("\033[38;5;%d;48;5;%dm", fg, bg); // set color
 			if (board[x][y] != 0)
 			{
 				uint32_t number = 1 << board[x][y];
-				uint8_t t = 7 - getNumberLength(number);
+				uint8_t t = 7 - getDigitCount(number);
 				printf("%*s%u%*s", t - t / 2, "", number, t / 2, "");
 			}
 			else
 			{
 				printf("   ·   ");
 			}
-			printf("\033[m"); // reset
+			printf("\033[m"); // reset all modes
 		}
 		printf("\n");
 		for (x = 0; x < SIZE; x++)
 		{
-			getColors(board[x][y], &fg, &bg);
+			getColors(board[x][y], scheme, &fg, &bg);
 			printf("\033[38;5;%d;48;5;%dm", fg, bg); // set color
 			printf("       ");
-			printf("\033[m"); // reset
+			printf("\033[m"); // reset all modes
 		}
 		printf("\n");
 	}
@@ -120,7 +123,7 @@ uint8_t findTarget(uint8_t array[SIZE], uint8_t x, uint8_t stop)
 	return x;
 }
 
-bool slideArray(uint8_t array[SIZE])
+bool slideArray(uint8_t array[SIZE], uint32_t *score)
 {
 	bool success = false;
 	uint8_t x, t, stop = 0;
@@ -143,7 +146,7 @@ bool slideArray(uint8_t array[SIZE])
 					// merge (increase power of two)
 					array[t]++;
 					// increase score
-					score += (uint32_t)1 << array[t];
+					*score += (uint32_t)1 << array[t];
 					// set stop to avoid double merge
 					stop = t + 1;
 				}
@@ -172,46 +175,46 @@ void rotateBoard(uint8_t board[SIZE][SIZE])
 	}
 }
 
-bool moveUp(uint8_t board[SIZE][SIZE])
+bool moveUp(uint8_t board[SIZE][SIZE], uint32_t *score)
 {
 	bool success = false;
 	uint8_t x;
 	for (x = 0; x < SIZE; x++)
 	{
-		success |= slideArray(board[x]);
+		success |= slideArray(board[x], score);
 	}
 	return success;
 }
 
-bool moveLeft(uint8_t board[SIZE][SIZE])
+bool moveLeft(uint8_t board[SIZE][SIZE], uint32_t *score)
 {
 	bool success;
 	rotateBoard(board);
-	success = moveUp(board);
+	success = moveUp(board, score);
 	rotateBoard(board);
 	rotateBoard(board);
 	rotateBoard(board);
 	return success;
 }
 
-bool moveDown(uint8_t board[SIZE][SIZE])
+bool moveDown(uint8_t board[SIZE][SIZE], uint32_t *score)
 {
 	bool success;
 	rotateBoard(board);
 	rotateBoard(board);
-	success = moveUp(board);
+	success = moveUp(board, score);
 	rotateBoard(board);
 	rotateBoard(board);
 	return success;
 }
 
-bool moveRight(uint8_t board[SIZE][SIZE])
+bool moveRight(uint8_t board[SIZE][SIZE], uint32_t *score)
 {
 	bool success;
 	rotateBoard(board);
 	rotateBoard(board);
 	rotateBoard(board);
-	success = moveUp(board);
+	success = moveUp(board, score);
 	rotateBoard(board);
 	return success;
 }
@@ -312,8 +315,6 @@ void initBoard(uint8_t board[SIZE][SIZE])
 	}
 	addRandom(board);
 	addRandom(board);
-	drawBoard(board);
-	score = 0;
 }
 
 void setBufferedInput(bool enable)
@@ -348,35 +349,39 @@ int test()
 {
 	uint8_t array[SIZE];
 	// these are exponents with base 2 (1=2 2=4 3=8)
+	// data holds per line: 4x IN, 4x OUT, 1x POINTS
 	uint8_t data[] = {
-		0, 0, 0, 1, 1, 0, 0, 0,
-		0, 0, 1, 1, 2, 0, 0, 0,
-		0, 1, 0, 1, 2, 0, 0, 0,
-		1, 0, 0, 1, 2, 0, 0, 0,
-		1, 0, 1, 0, 2, 0, 0, 0,
-		1, 1, 1, 0, 2, 1, 0, 0,
-		1, 0, 1, 1, 2, 1, 0, 0,
-		1, 1, 0, 1, 2, 1, 0, 0,
-		1, 1, 1, 1, 2, 2, 0, 0,
-		2, 2, 1, 1, 3, 2, 0, 0,
-		1, 1, 2, 2, 2, 3, 0, 0,
-		3, 0, 1, 1, 3, 2, 0, 0,
-		2, 0, 1, 1, 2, 2, 0, 0};
-	uint8_t *in, *out;
+	    0, 0, 0, 1, 1, 0, 0, 0, 0,
+	    0, 0, 1, 1, 2, 0, 0, 0, 4,
+	    0, 1, 0, 1, 2, 0, 0, 0, 4,
+	    1, 0, 0, 1, 2, 0, 0, 0, 4,
+	    1, 0, 1, 0, 2, 0, 0, 0, 4,
+	    1, 1, 1, 0, 2, 1, 0, 0, 4,
+	    1, 0, 1, 1, 2, 1, 0, 0, 4,
+	    1, 1, 0, 1, 2, 1, 0, 0, 4,
+	    1, 1, 1, 1, 2, 2, 0, 0, 8,
+	    2, 2, 1, 1, 3, 2, 0, 0, 12,
+	    1, 1, 2, 2, 2, 3, 0, 0, 12,
+	    3, 0, 1, 1, 3, 2, 0, 0, 4,
+	    2, 0, 1, 1, 2, 2, 0, 0, 4};
+	uint8_t *in, *out, *points;
 	uint8_t t, tests;
 	uint8_t i;
 	bool success = true;
+	uint32_t score;
 
-	tests = (sizeof(data) / sizeof(data[0])) / (2 * SIZE);
+	tests = (sizeof(data) / sizeof(data[0])) / (2 * SIZE + 1);
 	for (t = 0; t < tests; t++)
 	{
-		in = data + t * 2 * SIZE;
+		in = data + t * (2 * SIZE + 1);
 		out = in + SIZE;
+		points = in + 2 * SIZE;
 		for (i = 0; i < SIZE; i++)
 		{
 			array[i] = in[i];
 		}
-		slideArray(array);
+		score = 0;
+		slideArray(array, &score);
 		for (i = 0; i < SIZE; i++)
 		{
 			if (array[i] != out[i])
@@ -384,6 +389,10 @@ int test()
 				success = false;
 			}
 		}
+		if (score != *points)
+		{
+			success = false;
+		}
 		if (success == false)
 		{
 			for (i = 0; i < SIZE; i++)
@@ -395,7 +404,7 @@ int test()
 			{
 				printf("%d ", array[i]);
 			}
-			printf("expected ");
+			printf("(%d points) expected ", score);
 			for (i = 0; i < SIZE; i++)
 			{
 				printf("%d ", in[i]);
@@ -405,7 +414,7 @@ int test()
 			{
 				printf("%d ", out[i]);
 			}
-			printf("\n");
+			printf("(%d points)\n", *points);
 			break;
 		}
 	}
@@ -420,6 +429,7 @@ void signal_callback_handler(int signum)
 {
 	printf("         TERMINATED         \n");
 	setBufferedInput(true);
+	// make cursor visible, reset all modes
 	printf("\033[?25h\033[m");
 	exit(signum);
 }
@@ -427,6 +437,8 @@ void signal_callback_handler(int signum)
 int main(int argc, char *argv[])
 {
 	uint8_t board[SIZE][SIZE];
+	uint8_t scheme = 0;
+	uint32_t score = 0;
 	char c;
 	bool success;
 
@@ -443,6 +455,7 @@ int main(int argc, char *argv[])
 		scheme = 2;
 	}
 
+	// make cursor invisible, erase entire screen
 	printf("\033[?25l\033[2J");
 
 	// register signal handler for when ctrl-c is pressed
@@ -450,6 +463,7 @@ int main(int argc, char *argv[])
 
 	initBoard(board);
 	setBufferedInput(false);
+	drawBoard(board, scheme, score);
 	while (true)
 	{
 		c = getchar();
@@ -463,32 +477,32 @@ int main(int argc, char *argv[])
 		case 97:  // 'a' key
 		case 104: // 'h' key
 		case 68:  // left arrow
-			success = moveLeft(board);
+			success = moveLeft(board, &score);
 			break;
 		case 100: // 'd' key
 		case 108: // 'l' key
 		case 67:  // right arrow
-			success = moveRight(board);
+			success = moveRight(board, &score);
 			break;
 		case 119: // 'w' key
 		case 107: // 'k' key
 		case 65:  // up arrow
-			success = moveUp(board);
+			success = moveUp(board, &score);
 			break;
 		case 115: // 's' key
 		case 106: // 'j' key
 		case 66:  // down arrow
-			success = moveDown(board);
+			success = moveDown(board, &score);
 			break;
 		default:
 			success = false;
 		}
 		if (success)
 		{
-			drawBoard(board);
-			usleep(150000);
+			drawBoard(board, scheme, score);
+			usleep(150 * 1000); // 150 ms
 			addRandom(board);
-			drawBoard(board);
+			drawBoard(board, scheme, score);
 			if (gameEnded(board))
 			{
 				printf("         GAME OVER          \n");
@@ -503,7 +517,7 @@ int main(int argc, char *argv[])
 			{
 				break;
 			}
-			drawBoard(board);
+			drawBoard(board, scheme, score);
 		}
 		if (c == 'r')
 		{
@@ -512,12 +526,14 @@ int main(int argc, char *argv[])
 			if (c == 'y')
 			{
 				initBoard(board);
+				score = 0;
 			}
-			drawBoard(board);
+			drawBoard(board, scheme, score);
 		}
 	}
 	setBufferedInput(true);
 
+	// make cursor visible, reset all modes
 	printf("\033[?25h\033[m");
 
 	return EXIT_SUCCESS;
